using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid.Grouping;


namespace GGC_AutoExtendColumn
{
	/// <summary>
	/// Wraps code required for auto-adjusting colwidths into one class
	/// </summary>
	public class GridColSizeHelper
	{
        private GridGroupingControl grid = null;
		private GridColSizeBehavior _colSizeBehavior;
		private double[] colRatios = null;

		public GridColSizeHelper()
		{
			
		}

		public void WireGrid(GridGroupingControl grid)
		{
			if(this.grid != grid)
			{
				if(this.grid != null)
					UnwireGrid();
			
				this.grid = grid;				

				//save original col ratios
				colRatios = new double[this.grid.TableModel.ColCount + 1];
				double dWidth = this.grid.ClientSize.Width;
                for(int col = 0; col <= this.grid.TableModel.ColCount; ++ col)
				{
					colRatios[col] = this.grid.TableModel.ColWidths[col] / dWidth;
				}
                this.grid.TableModel.QueryColWidth += new GridRowColSizeEventHandler(TableModel_QueryColWidth);
                this.grid.TableModel.ColWidthsChanged += new GridRowColSizeChangedEventHandler(TableModel_ColWidthsChanged);
                this.grid.TableControl.ResizingColumns += new GridResizingColumnsEventHandler(TableControl_ResizingColumns);

                this.grid.GetTableModel("MyChildTable").QueryColWidth += new GridRowColSizeEventHandler(GridColSizeHelper_QueryColWidth);
                this.grid.GetTableModel("MyGrandChildTable").QueryColWidth += new GridRowColSizeEventHandler(GridColSizeHelper_QueryColWidth);
            }
		}

        void GridColSizeHelper_QueryColWidth(object sender, GridRowColSizeEventArgs e)
        {
            switch (_colSizeBehavior)
            {
                case GridColSizeBehavior.FillLeftColumn:
                    GridTableModel Model = sender as GridTableModel;
                    string Name = Model.Table.TableDescriptor.Name;   
                    GridTableControl tc = this.grid.GetTableControl(Name);  

                    int FrozenColIndex = tc.GetFirstScrollableCol();

                    if (e.Index == FrozenColIndex)
                    {
                        int ParentIndentColumn =  Name != "MyGrandChildTable" ? 0 : 3;
                        int indentCols = (Model.GetColumnIndentCount() + ParentIndentColumn)  * this.grid.TableOptions.IndentWidth;

                        int leftPiece =  Model.ColWidths.GetTotal(0, FrozenColIndex - 1);
                        int rightPiece = Model.ColWidths.GetTotal(FrozenColIndex + 1 , Model.ColCount);
                        e.Size = this.grid.ClientSize.Width - leftPiece - rightPiece - indentCols;
                        e.Handled = true;
                    }
                    break;
                default:
                    break;
            }
        }

        void TableControl_ResizingColumns(object sender, GridResizingColumnsEventArgs e)
        {
              
           if (_colSizeBehavior == GridColSizeBehavior.FillRightColumn && e.Columns.Right == this.grid.TableModel.ColCount)
                e.Cancel = true;
            else if (_colSizeBehavior == GridColSizeBehavior.FillLeftColumn && e.Columns.Left == this.grid.TableModel.Cols.HeaderCount + 1)
                e.Cancel = true;
        }
        private bool inColWidthsChanged = false;
        void TableModel_ColWidthsChanged(object sender, GridRowColSizeChangedEventArgs e)
        {
            if (this.inColWidthsChanged)
                return;
            inColWidthsChanged = true;


            inColWidthsChanged = false;
        }

        void TableModel_QueryColWidth(object sender, GridRowColSizeEventArgs e)
        {
            switch(_colSizeBehavior)
			{
				case GridColSizeBehavior.FillRightColumn:
					if(e.Index == this.grid.TableDescriptor.VisibleColumns.Count)
					{
						e.Size = this.grid.ClientSize.Width - this.grid.TableModel.ColWidths.GetTotal(0, this.grid.TableDescriptor.VisibleColumns.Count - 1);
						e.Handled = true;
					}
					break;

                case GridColSizeBehavior.FillLeftColumn:
                     int FrozenColIndex = this.grid.TableControl.GetFirstScrollableCol();
                    if (e.Index == FrozenColIndex)
                    {
                        int leftPiece = this.grid.TableModel.ColWidths.GetTotal(0, FrozenColIndex - 1);//this.grid.TableModel.Cols.FrozenCount);
                        
                        int rightPiece = this.grid.TableModel.ColWidths.GetTotal(FrozenColIndex + 1, this.grid.TableModel.ColCount);
                        e.Size = this.grid.ClientSize.Width - leftPiece - rightPiece;
                        e.Handled = true;
                    }
                    break;

                case GridColSizeBehavior.Fill2ndColumn:
					if(e.Index == this.grid.TableDescriptor.VisibleColumns.Count)
					{
						e.Size = this.grid.ClientSize.Width - this.grid.TableModel.ColWidths.GetTotal(0, this.grid.TableDescriptor.VisibleColumns.Count - 1);
						e.Handled = true;
					}
					break;					

                    

				default:
					break;
			}
        }

		public void UnwireGrid()
		{
            this.grid.TableModel.QueryColWidth -= new GridRowColSizeEventHandler(TableModel_QueryColWidth);
            this.grid.TableModel.ColWidthsChanged -= new GridRowColSizeChangedEventHandler(TableModel_ColWidthsChanged);
            this.grid.TableControl.ResizingColumns -= new GridResizingColumnsEventHandler(TableControl_ResizingColumns);

            this.grid.GetTableModel("MyChildTable").QueryColWidth -= new GridRowColSizeEventHandler(GridColSizeHelper_QueryColWidth);
            this.grid.GetTableModel("MyGrandChildTable").QueryColWidth -= new GridRowColSizeEventHandler(GridColSizeHelper_QueryColWidth);
       
                
			this.grid = null;
		}	
		
		public GridColSizeBehavior ColSizeBehavior
		{
			get{return _colSizeBehavior;}
			set{ _colSizeBehavior = value;}
		}
		public enum GridColSizeBehavior 
		{
			None = 0,
			FillRightColumn,
			FillLeftColumn,
            Fill2ndColumn
			
		}
	}
}
